from django.conf import settings
from django.contrib.auth.models import Group, Permission
from django.core.management.base import BaseCommand
from django.db.models import Q, QuerySet

PERMISSIONS = {
    settings.PRIVILEGED_GROUP: {
        "app": Permission.objects.filter(
            ~Q(codename__startswith="edit"),
            content_type__app_label="program",
        ),
        "custom_add": Permission.objects.filter(
            codename__startswith="add__",
            content_type__model="playlist",
        ),
        "custom_display": Permission.objects.filter(codename="display__show__internal_note"),
        "custom_update": Permission.objects.filter(
            codename__startswith="update",
            content_type__model__in=["host", "note", "show"],
        ),
    },
    settings.ENTITLED_GROUPS[0]: {
        "default_note_notelink": Permission.objects.filter(
            ~Q(codename__startswith="create"),
            ~Q(codename__startswith="edit"),
            ~Q(codename__startswith="update"),
            content_type__model__in=["note", "notelink"],
        ),
    },
    settings.ENTITLED_GROUPS[1]: {
        "extra_change": Permission.objects.filter(
            codename__startswith="change",
            content_type__model__in=["host", "note", "show"],
        ),
        "custom_add": Permission.objects.filter(
            ~Q(codename__startswith="add__m3ufile"),
            codename__startswith="add__",
            content_type__model="playlist",
        ),
        "custom_edit": Permission.objects.filter(
            codename__startswith="edit",
            content_type__model__in=["host", "note", "show"],
        ),
    },
}


class Command(BaseCommand):
    help = "add permissions"

    def add_permissions(self, group: Group, permissions: QuerySet, name: str):
        self.stdout.write(f"Adding {name} permissions to the '{group.name}' group... ", ending="")
        self.stdout.flush()
        group.permissions.add(*permissions)
        self.stdout.write(self.style.SUCCESS(str(len(permissions))))

    def handle(self, *args, **options):
        for group_name in PERMISSIONS:
            group = Group.objects.get(name=group_name)

            for name, permissions in PERMISSIONS[group_name].items():
                self.add_permissions(group, permissions, name)
